﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers.Views {
  public partial class SuccessorConfigurator : Form {

    public HeuristicOptimizationEngineAlgorithm Content { get; set; }
    private CombinedOperator currentSuccessor = null;
    private Type operatorType = null;

    public SuccessorConfigurator() {
      InitializeComponent();
    }

    private void chooseSuccessorButton_Click(object sender, System.EventArgs e) {
      using (TypeSelectorDialog dialog = new TypeSelectorDialog()) {
        dialog.Caption = "Select Successor";
        dialog.TypeSelector.Caption = "Available Successor Operators";
        dialog.TypeSelector.Configure(typeof(CombinedOperator), false, true);

        if (dialog.ShowDialog() == DialogResult.OK) {
          currentSuccessor = (CombinedOperator)dialog.TypeSelector.CreateInstanceOfSelectedType();
          successorTextBox.Text = currentSuccessor.ToString();
        }
      }
    }

    private void okButton_Click(object sender, System.EventArgs e) {
      ParametrizeOperators();
      Close();
    }

    private void cancelButton_Click(object sender, System.EventArgs e) {
      Close();
    }

    private void chooseOperatorTypeButton_Click(object sender, System.EventArgs e) {
      using (TypeSelectorDialog dialog = new TypeSelectorDialog()) {
        dialog.Caption = "Select Operator Type";
        dialog.TypeSelector.Caption = "Available Operator Types";
        dialog.TypeSelector.Configure(typeof(IOperator), true, true);

        if (dialog.ShowDialog() == DialogResult.OK) {
          operatorType = dialog.TypeSelector.SelectedType;
          operatorNameTextBox.Text = operatorType.ToString();
        }
      }
    }

    private void ParametrizeOperators() {
      foreach (var op in Content.Problem.Operators.OfType<SingleSuccessorOperator>()) {
        if (op.GetType().GetInterfaces().Any(x => x == operatorType)) {
          op.Successor = (IOperator)currentSuccessor.Clone(new Cloner());
        }
      }
    }
  }
}
